/*****************************************************************************/
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/

#include "cf_builtins.hh"
#include "cf_exception.hh"
#include <string>
#include <list>
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>

using namespace std;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_ClearBox

string CF_ClearBox::Call(vector<string>& args)
{
	CF_ToolbarWidget* w = widgets[args[0]];
	if(w != NULL)
	{
		CF_ToolbarWidget::Type type = w->GetType();
		if(type == CF_ToolbarWidget::TEXT)
		{
			CF_Text* t = reinterpret_cast<CF_Text*>(w);
			t->SetEntry("", false);
		}//if(type == CF_ToolbarWidget::TEXT)
		else if (type == CF_ToolbarWidget::COMBO)
		{
			CF_Combo* c = reinterpret_cast<CF_Combo*>(w);
			c->SetEntry("", false);
		}//else if (type == CF_ToolbarWidget::COMBO)
	}//if(w != NULL)
	
	return "";
}//string CF_ClearBox::Call(vector<string>& args)


//END CF_ClearBox


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_ClearList

string CF_ClearList::Call(vector<string>& args)
{
	CF_ToolbarWidget* w = widgets[args[0]];
	if(w != NULL)
	{
		CF_ToolbarWidget::Type type = w->GetType();
		if (type == CF_ToolbarWidget::COMBO)
		{
			CF_Combo* c = reinterpret_cast<CF_Combo*>(w);
			c->ClearList();
		}//else if (type == CF_ToolbarWidget::COMBO)
	}//if(w != NULL)
	
	return "";	
}//string CF_ClearList::Call(vector<string>& args)

//END CF_ClearList


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_AddToList


string CF_AddToList::Call(vector<string>& args)
{
	//get widget
	CF_ToolbarWidget* w = widgets[args[0]];
	if(w != NULL)
	{
		if(w->GetType() == CF_ToolbarWidget::COMBO)
		{
			CF_Combo* c = reinterpret_cast<CF_Combo*>(w);
			
			//parse addd string
			list<string> add;
			string curAdd;
			for(size_t pos = 0; pos < args[1].length(); ++pos)
			{
				switch(args[1][pos])
				{
				case ' ':
				case '\t':
				case '\n':
					add.push_back(curAdd);
					curAdd.erase();
					break;
					
				default:
					curAdd += args[1][pos];
					break;
				}//switch(args[1][pos])
			}//for(size_t pos = 0; pos < args[1].length(); ++pos)
			if(!curAdd.empty())
				add.push_back(curAdd);
				
			//add strings
			c->AddToList(add);
		}//if(w->GetType() == CF_ToolbarWidget::COMBO)
	}//if(w != NULL)
	
	return "";
}//string CF_AddToList::Call(vector<string>& args)


//END CF_AddToList


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_SetBox

string CF_SetBox::Call(vector<string>& args)
{
	CF_ToolbarWidget* w = widgets[args[0]];
	if(w != NULL)
	{
		bool activate = true;
		if(args.size() > 2)
		{
			if((args[2] == "") || (args[2] == "false"))
				activate = false;
		}//if(args.size() > 2)
		CF_ToolbarWidget::Type type = w->GetType();
		if(type == CF_ToolbarWidget::TEXT)
		{
			CF_Text* t = reinterpret_cast<CF_Text*>(w);
			t->SetEntry(args[1], activate);
		}//if(type == CF_ToolbarWidget::TEXT)
		else if (type == CF_ToolbarWidget::COMBO)
		{
			CF_Combo* c = reinterpret_cast<CF_Combo*>(w);
			c->SetEntry(args[1], activate);
		}//else if (type == CF_ToolbarWidget::COMBO)
	}//if(w != NULL)
	
	return "";
}//string CF_SetBox::Call(vector<string>& args)

//END CF_SetBox


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_SetToggle

string CF_SetToggle::Call(vector<string>& args)
{
	CF_ToolbarWidget* w = widgets[args[0]];
	if(w != NULL)
	{
		if(w->GetType() == CF_ToolbarWidget::TOGGLE)
		{
			CF_Toggle* t = reinterpret_cast<CF_Toggle*>(w);
			t->SetState(args[1]);
		}//if(type == CF_ToolbarWidget::TEXT)
	}//if(w != NULL)
	
	return "";	
}//string CF_SetToggle::Call(vector<string>& args)

//END CF_SetToggle


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_Shell

string CF_Shell::Call(vector<string>& args)
{
	//open pipe
	FILE* pipe = popen(args[0].c_str(), "r");
	if(pipe == NULL)
		throw CF_Exception("CF_Shell::Call()", string("Couldn't open pipe to command:\n") + args[0]);
	
	//read output
	char buffer[256];
	string output;
	while(fgets(buffer, sizeof(buffer), pipe))
		output += buffer;
		
	//close pipe
	if(pclose(pipe) != 0)
		throw CF_Exception("CF_Shell::Call()", "Error closing pipe");
	
	return output; 
}//string CF_Shell::Call(vector<string>& args)

//END CF_Shell




//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_SetVar

string CF_SetVar::Call(vector<string>& args)
{
	//get var
	CF_ScriptVar* var = mgr.GetVar(args[0]);
	if(var != NULL)
	{
		var->SetValue(args[1]);
	}//if(var != NULL)
	
	return "";
}//string CF_SetVar::Call(vector<string>& args)

//END CF_SetVar


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN CF_GetFileName

CF_GetFileName::CF_GetFileName(CF_VarMgr& mgrIn, map<const string, CF_ToolbarWidget*>& widgetsIn) :
	 CF_CallBack("GetFilename", 0), fileSelect("Select File"), varMgr(mgrIn), widgetMap(widgetsIn)
{
	connect_to_method(fileSelect.get_ok_button()->clicked, this, &CF_GetFileName::OKClicked);
	connect_to_method(fileSelect.get_cancel_button()->clicked, this, &CF_GetFileName::CancelClicked);
	connect_to_method(fileSelect.delete_event, this, &CF_GetFileName::DeleteEvent);
	
	widget = NULL;
	var = NULL;
	activate = true;
}//CF_GetFileName()

string CF_GetFileName::Call(vector<string>& args)
{
	//init args
	fileSelect.set_title("Select File...");
	var = NULL;
	widget = NULL;
	activate = false;
	
	//set up file selection title
	if(args.size() > 0)
		fileSelect.set_title(args[0]);
		
	//set up var
	if(args.size() > 1)
		var = varMgr.GetVar(args[1]);
	
	//set up widget
	if(args.size() > 2)
	{
		widget = widgetMap[args[2]];
		if(widget != NULL)
		{
			CF_ToolbarWidget::Type type = widget->GetType();
			if((type == CF_ToolbarWidget::BUTTON) || (type == CF_ToolbarWidget::TOGGLE))
				widget = NULL;
		}//if(widget != NULL)
	}//if(args.size() > 1)
	else
	{
		widget = NULL;
	}//else
	
	//check activation
	activate = true;	//defaults to true
	if(args.size() > 3)
	{
		if((args[3] == "") || (args[3] == "false"))
			activate = false;
	}//if(args.size() > 2)
		
	fileSelect.set_modal(true);	
	fileSelect.show();
	canceled = false;
	gtk_main();
	
	if(!canceled)
	{
		//get file name
		string name = fileSelect.get_filename();

		//store in var
		if(var != NULL)
			var->SetValue(name);

		//store in widget
		if(widget != NULL)
		{
			CF_ToolbarWidget::Type type = widget->GetType();
			if(type == CF_ToolbarWidget::COMBO)
			{
				CF_Combo* c = reinterpret_cast<CF_Combo*>(widget);
				c->SetEntry(name, activate);
			}//if(type == CF_ToolbarWidget::COMBO)
			else if(type == CF_ToolbarWidget::TEXT)
			{
				CF_Text* c = reinterpret_cast<CF_Text*>(widget);
				c->SetEntry(name, activate);
			}//if(type == CF_ToolbarWidget::COMBO)
		}//if(widget != NULL)
		return name;	
	}//if(!canceled)
	
	return "";
}//string Call(vector<string>& args)

void CF_GetFileName::OKClicked()
{
	fileSelect.hide();
	gtk_main_quit();
}//void OKClicked()

void CF_GetFileName::CancelClicked()
{
	canceled = true;
	fileSelect.hide();
	gtk_main_quit();
}//void CF_GetFileName::CancelClicked()

gint CF_GetFileName::DeleteEvent(GdkEventAny *ev)
{
	canceled = true;
	fileSelect.hide();
	gtk_main_quit();
	return TRUE;
}//gint CF_GetFileName::DeleteEvent(GdkEventAny *ev)


//END CF_GetFileName


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN GLOBALS

//END GLOBALS
