/*****************************************************************************/
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/

#ifndef CF_BUILTINS_HH
#define CF_BUILTINS_HH

#include "cf_callback.hh"
#include "cf_toolbarwidget.hh"
#include "cf_scriptvar.hh"
#include <string>
#include <map>
#include <gtk--/fileselection.h>
#include <gtk--/main.h>

using namespace std;


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_ClearBox
//	-clears contents of a text box or a combo box (text box part only)

class CF_ClearBox : public CF_CallBack
{
public:
	CF_ClearBox(map<const string, CF_ToolbarWidget*>& widgetsIn): CF_CallBack("ClearBox", 1), widgets(widgetsIn) {}
	
	string Call(vector<string>& args);

private:
	map<const string, CF_ToolbarWidget*>& widgets;
};//class CF_ClearBox


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_ClearList
//	-clears a combo box list

class CF_ClearList : public CF_CallBack
{
public:
	CF_ClearList(map<const string, CF_ToolbarWidget*>& widgetsIn): CF_CallBack("ClearList", 1), widgets(widgetsIn) {}

	string Call(vector<string>& args);

private:
	map<const string, CF_ToolbarWidget*>& widgets;
};//class CF_ClearList


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_AddToList
//	-Adds strrings to combo box list

class CF_AddToList : public CF_CallBack
{
public:
	CF_AddToList(map<const string, CF_ToolbarWidget*>& widgetsIn): CF_CallBack("AddToList", 2), widgets(widgetsIn) {}

	string Call(vector<string>& args);
	
private:
	map<const string, CF_ToolbarWidget*>& widgets;
};//class CF_AddToList


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_SetBox
//	-Sets the given entry box to the given string

class CF_SetBox : public CF_CallBack
{
public:
	CF_SetBox(map<const string, CF_ToolbarWidget*>& widgetsIn): CF_CallBack("SetBox", 2), widgets(widgetsIn) {}

	string Call(vector<string>& args);
	
private:
	map<const string, CF_ToolbarWidget*>& widgets;
};//class CF_SetBox


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_SetToggle
//	-sets the state of a toggle

class CF_SetToggle : public CF_CallBack
{
public:
	CF_SetToggle(map<const string, CF_ToolbarWidget*>& widgetsIn): CF_CallBack("SetToggle", 2), widgets(widgetsIn) {}

	string Call(vector<string>& args);
	
private:
	map<const string, CF_ToolbarWidget*>& widgets;
};//class CF_SetToggle


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_Shell

class CF_Shell : public CF_CallBack
{
public:
	CF_Shell() : CF_CallBack("Shell", 1) {}

	string Call(vector<string>& args);
};//class CF_Shell


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_SetVar

class CF_SetVar : public CF_CallBack
{
public:
	CF_SetVar(CF_VarMgr& mgrIn) : CF_CallBack("SetVar", 2), mgr(mgrIn) {}

	string Call(vector<string>& args);
	
private:
	CF_VarMgr& mgr;
};//class CF_SetVar

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_GetFileName

class CF_GetFileName : public CF_CallBack
{
public:
	CF_GetFileName(CF_VarMgr& mgrIn, map<const string, CF_ToolbarWidget*>& widgetsIn);

	string Call(vector<string>& args);
	
	void OKClicked();
	void CancelClicked();
	gint DeleteEvent(GdkEventAny *ev);

private:
	Gtk_FileSelection fileSelect;
	CF_ScriptVar* var;
	CF_ToolbarWidget* widget;
	CF_VarMgr& varMgr;
	map<const string, CF_ToolbarWidget*>& widgetMap;
	bool activate;
	bool canceled;
};//class CF_GetFileName

#endif
