/*****************************************************************************/
//  cf_callback.cc
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


//////////////////////////////////////////////////
// cf_callback.cc
//
//////////////////////////////////////////////////


#include "cf_callback.hh"
#include "cf_exception.hh"


//////////////////////////////////////////////////
//BEGIN CF_CallBack


//END CF_CallBack


/////////////////////////////////////////////////////
//BEGIN CF_CommandMgr

//CF_CommandMgr::RegisterFunction: registers a function
//	function: frunction to register
//returns: true if no name collision
bool CF_CommandMgr::RegisterFunction(CF_CallBack* function)
{
	if(functions[function->GetName()] != NULL)
		return false;
	functions[function->GetName()] = function;
	return true;
}//bool CF_CommandMgr::RegisterFunction(CF_CallBack* function)

//CF_CommandMgr::CallFunction: Calls the given function
//	name: function to call
//	args: args for function
//returns: output from function
string CF_CommandMgr::CallFunction(const string& name, vector<string>& args)
{
	if(functions[name] == NULL)
	{
		string msg = string("Non-existent function called: ") + name;
		throw CF_Exception("CF_CommandMgr::CallFunction()", msg); 
	}//if(functions[name] == NULL)
	return functions[name]->Call(args);
}//string CF_CommandMgr::CallFunction(const string& name, vector<string>& args)


//END CF_CommandMgr


/////////////////////////////////////////////////////
//BEGIN CF_Option


//END CF_Option



/////////////////////////////////////////////////////
//BEGIN CF_OptionsMgr

bool CF_OptionMgr::AddOption(const string& name)
{
	if(options[name] != NULL)
		return false;
	CF_Option* newOpt = new CF_Option(name);
	CF_Assert(newOpt != NULL, "CF_OptionsMgr::AddOpt()");
	options[name] = newOpt;
	return true;
}//bool CF_OptionsMgr::AddOption(CF_Option* option)


bool CF_OptionMgr::QueryOption(const string& name)
{
	if(options[name] == NULL)
		return false;
	
	return options[name]->Found();
}//bool CF_OptionsMgr::QueryOption(const string& name) 

string CF_OptionMgr::GetArg(const string& name)
{
	if(options[name] == NULL)
		return "";
	
	return options[name]->GetArg();
}//const string& GetArg(const string& name)

bool CF_OptionMgr::SetOptionFound(const string& name, const string& arg)
{
	if(options[name] == NULL)
		return false;
	
	options[name]->SetFound(arg);	
	return true;
}//bool SetOptionFound(const string& name, const string& arg)

CF_OptionMgr::~CF_OptionMgr()
{
	for(map<const string, CF_Option*>::iterator it = options.begin(); it != options.end(); ++it)
	{
		if((*it).second != NULL)
			delete (*it).second;
	}//for(map<const string, CF_Option*>::iterator it = options.begin(); it != options.end(); ++it)
}//CF_OptionsMgr::~CF_OptionsMgr()

//END CF_OptionsMgr


//////////////////////////////////////////////////
//BEGIN GLOBALS


//END GLOBALS


