/*****************************************************************************/
//  cf_exception.hh
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// cf_exception.hh
//	-CF_Exception: exception class for control freak
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


#ifndef CF_EXCEPTION_HH
#define CF_EXCEPTION_HH

#include <string>
#include <iostream>

using namespace std;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//CF_Exception

/**General control freak exception class.
	Contains name of module where exception was generated and exception message.  DisplayMsg()
	displays this info.
*/
class CF_Exception
{
public:
	/**Creates an exception object
		@param moduleIn Name of module that is throwing exception
		@param msgIn Message to display when exception is caught
	*/
	CF_Exception(const string& moduleIn, const string& msgIn): module(moduleIn), msg(msgIn) {}
	
	/**Displays the exception message and module
		@param exitVal If non-zero program will exit with this code
	*/
	void DisplayMsg(int exitVal);

private:
	string module;
	string msg;
};

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_ParseException

/**Control freak exception specialized to be thrown when a parse error occurs in a script.
*/
class CF_ParseException 
{
public:
	
	/**Creates a parse exception object
		@param moduleIn Name of module that is throwing exception
		@param msgIn Message to display when exception is caught
	*/
	CF_ParseException(const string& moduleIn, const string& msgIn) : module(moduleIn), msg(msgIn) {}
	
	/**Displays exception message along with file name and line number where error occured
	 	(only displays module in debug builds)
		@param file Name of file where error occured
		@param lineNum Line number where error occured
		@param exitVal If non-zero program will exit with this value
	*/
	void DisplayMsg(const string& file, int lineNum, int exitVal);

private:
	string module;
	string msg;
};//class CF_ParseException


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//Assert macro - throws CF_Exception if false (not removed when doing a release build)

/**Throws CF_Exception if condition is false (not removed in release builds)
	@param condition Condition to test
	@param module Module where test is performed
	@param msg Exception message
*/
inline void CF_Assert(bool condition, const string& module, const string& msg = "Memory allocation error")
{
	if(!condition)
		throw CF_Exception(module, msg);
}//inline void CF_Assert(int condition)

#endif
