/*****************************************************************************/
//  cf_scriptvar.hh
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// cf_scriptvar.hh
//	-CF_ScriptVar: script variable base class
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


#ifndef CF_SCRIPTVAR_HH
#define CF_SCRIPTVAR_HH

#include <string>
#include <map>

using namespace std;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//CF_ScriptVar

///Base class for script variables
class CF_ScriptVar
{
public:	
	/**Creates a script var
		@param nameIn Name for var
		@param initValue Initial Value for var
	*/
	CF_ScriptVar(const string& nameIn, const string& initValue = "") : name(nameIn) {SetValue(initValue);}
	
	/**Creates a script var
		@param nameIn Name for var
		@param initValue Initial Value for var
	*/
	CF_ScriptVar(const string& nameIn, bool initValue = false) : name(nameIn) {SetValue(initValue);}

	/**Gets name of var
		@return Name of var
	*/
	const string& GetName() {return name;}	
	
	/**Gets var's value
		@return String version of var value
	*/
	const string& GetStringValue() {return sValue;}
	
	/**Gets var's value
		@return Boolean version of var's value
	*/
	bool GetBoolValue() {return bValue;}
	
	/**Sets the var's value
		@param valIn String version of var value
	*/
	void SetValue(const string& valIn);
	
	
	/**Sets the var's value
		@param valIn Boolean version of var value
	*/
	void SetValue(bool valIn);

protected:
	string name;
	string sValue;
	bool bValue;
};


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// class CF_VarMgr

/**Script variable manager class.
*/
class CF_VarMgr 
{
public:
	CF_VarMgr() {}
	
	//Deletes all registered vars
	~CF_VarMgr();
	
	/**Creates a new script var
		@param name Name for var
		@param initValue Inital value for var
		@return True if no name collisions with existing vars
	*/
	bool AddVar(const string &name, const string& initValue); 
	
	/**Creates a new script var
		@param name Name for var
		@param initValue Inital value for var
		@return True if no name collisions with existing vars
	*/
	bool AddVar(const string &name, bool initValue);
	
	/**Gets a var
		@param name Name of var to get
		@return Pointer to var, NULL if var doesn't exist
	*/
	CF_ScriptVar* GetVar(const string& name)	{return vars[name];}
	
	typedef map<const string, CF_ScriptVar*>::iterator iterator;
	///Returns iterator at beginning of variables
	iterator begin() {return vars.begin();}
	///Returns iterator at end of variables
	iterator end()	{return vars.end();}
	
private:
	map<const string, CF_ScriptVar*> vars;
};//class CF_VarMgr



#endif
