/*****************************************************************************/
//  cf_widget.hh
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// cf_widget.hh
//	-CF_Widget class: container for toolbars created by control freak
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


#ifndef CF_WIDGET_HH
#define CF_WIDGET_HH

#include <gtk--/toolbar.h>
#include <gtk--/box.h>
#include <gtk--/window.h>
#include <string>
#include <map>
#include <list>
#include <vector>
#include <iostream>
#include "cf_toolbarwidget.hh"
#include "cf_scriptvar.hh"
#include "cf_callback.hh"
#include "cf_command.hh"
#include "cf_builtins.hh"

using namespace std;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//CF_Widget

/**The Top level control freak widget class.
		To create a control freak widget instantiate this class using the desired script file in
		the constructor.
*/
class CF_Widget
{
public:
	/**Enumerates toolbar orientation
		<ll>
		<li><b>VERT</b> - vertically oriented toolbar
		<li><b>HORZ</b> - horizontally oriented toolbar
	*/
	enum Orientation {VERT, HORZ};
	
	/**Creates a control freak widget
		@param fileName Name of the script file to parse when building widget
		@param win Ref to window that widget will be placed in
		@param cmdMgr Pointer to command manager to use, if none provided a manager with default commands will be used
		@param optionsMgr Pointer to command line options manager
	*/
	CF_Widget(const string& fileName, Gtk_Window& win, CF_CommandMgr* cmdMgr = NULL, CF_OptionMgr* optionsMgr = NULL);
	
	//Deletes all widgets and script vars
	~CF_Widget();
	
	/**Calls the given function
		@param name Name of function to call
		@param args Arguments for function
		@return Retun value of function
	*/
	string CallFunction(const string& name, vector<string>& args)	{return functions->CallFunction(name, args);}
	
	/**Gets the named widget
		@param name Name of widget to get
		@return Pointer to widget, NULL if widget doesn't exist
	*/
	CF_ToolbarWidget* GetWidget(const string& name);
	
	/**Gets the given script var
		@param name Name of script var to get
		@return Pointer to script var, NULL if var doesn't exist
	*/
	CF_ScriptVar* GetScriptVar(const string& name)	{return vars.GetVar(name);}	
	
	/**Gets the gtk-- widget that all toolbars are packed into
		@return GtkWidget that is packable and contains all toolbars
	*/
	Gtk_Widget& GetPackableWidget() {return *box;}	//gets the widget that all toolbars are packed into
	
	///The Echo "built-in" function
	class CF_EchoCmd : public CF_CallBack
	{
	public:
		///Creates an echo callback
		CF_EchoCmd() : CF_CallBack("Echo", 1) {}
		
		/**Echos the commands args to std out
			@param args Stringg to echo to std out
			@return Empty string
		*/
		string Call(vector<string>& args);
	}echo;//class CF_EchoCmd

	
private:
	bool AddWidget(CF_ToolbarWidget* widget);
	string ReadLine(ifstream& file, int& lineNum);	//reads a line from file, removes comments and extraneous
									//whitespace in line, concatenates lines if "\" used

	list<Gtk_Widget*> toolbars;	//list of toolbars and handle boxes, used by destructor to remove toolbars
	list<Gtk_Box*> boxes;	//list of boxes for packing toolbars
	Gtk_Widget* box;	//top level box that toolbar boxes are packed into
	map<const string, CF_ToolbarWidget*> widgets;	//widget map
	CF_VarMgr vars;	//script variable map
	CF_CommandMgr* functions;
	bool delFunctions;
	CF_OptionMgr* options;
	bool delOptions;
	string scriptName;	//name of script that widget was built from
	list<string> save;	//list of widgets that need to be saved/loaded
	list<string> saveVars;	//list of vars whose state should be saved/loaded
	CEList startupCmds;
	CEList shutdownCmds;
	
	//built-in commands
	CF_ClearBox clearBox_bi;
	CF_ClearList clearList_bi;
	CF_AddToList addToList_bi;
	CF_SetBox setBox_bi;
	CF_SetToggle setToggle_bi;
	CF_Shell shell_bi;
	CF_SetVar setVar_bi;
	CF_GetFileName getFilename_bi;
};


#endif
