/*****************************************************************************/
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


#include "pathinfo.hh"
#include <stdlib.h>
#include <fstream>

using namespace std;

PathInfo& PathInfo::GetInstance()
{
	static PathInfo info;
	
	return info;
}//PathInfo& PathInfo::GetInstance()

void PathInfo::AddPaths(const string& pathStr)
{
	
	string curPath;
	for(size_t pos = 0; pos < pathStr.length(); ++pos)
	{
		switch(pathStr[pos])
		{
		case ':':
			if(curPath[curPath.length() - 1] != '/')
				curPath += '/';
			paths.push_front(curPath);
			curPath.erase();
			break;
			
		default:
			curPath += pathStr[pos];
		}//switch(pathStr[pos])
	}//for(size_t pos = 0; pos < pathStr.length(); ++pos)
	if(!curPath.empty())
	{
		if(curPath[curPath.length() - 1] != '/')
			curPath += '/';
		paths.push_front(curPath);
	}//if(!curPath.empty())
}//void PathInfo::AddPaths(const string& pathStr)

string PathInfo::BuildPathName(const string& filename)
{
	//if path has leading "/" then absolute, only check that path
	if(filename[0] == '/')
	{
		ifstream file(filename.c_str(), ios::nocreate);
		if(!file)
			return "";
		else
		{
			file.close();
			return filename;
		}//else
	}//if(filename[0] == '/')
	
	//try current dir
	ifstream file(filename.c_str(), ios::nocreate);
	if(!file){}
	else
	{
		file.close();
		return filename;
	}//else
	
	//try all directories in path
	for(list<string>::iterator it = paths.begin(); it != paths.end(); ++it)
	{
		string pathName = (*it) + filename;
		file.open(pathName.c_str(), ios::nocreate);
		if(!file){}
		else
		{
			file.close();
			return pathName;
		}//else
	}//for(list<string>::iterator it = paths.begin(); it != paths.end(); ++paths)
	
	//didn't find file anywhere
	return "";
}//string PathInfo::BuildPathName(const string& filename)

string PathInfo::GetPathString()
{
	string pathStr = "./";
	for(list<string>::iterator it = paths.begin(); it != paths.end(); ++it)
		pathStr += string(":") + (*it);

	return pathStr;
}//string PathInfo::GetPathString()

PathInfo::PathInfo()
{
	//Note: using AddPaths so have to reverse order that paths are added in (AddPaths prepends paths)	
	
	//add install directory to paths
	installDir = INSTALL_DIR_BASE;
	installDir += "/share/";
	installDir += APP_NAME;
	installDir += "/";
	AddPaths(installDir);
	
	//add home dir to paths
	char* homePtr = getenv("HOME");
	if(homePtr != NULL)
	{
		homeDir = homePtr;
		appHomeDir = homeDir + "/.%A/";
		appHomeDir.replace(appHomeDir.find("%A"), 2, APP_NAME);
		AddPaths(appHomeDir);
	}//if(homePtr != NULL)
	
	//add env var path to paths
	char* envPathPtr = getenv(PATH_ENV_VAR);
	if(envPathPtr != NULL)
	{
		AddPaths(envPathPtr);
	}//if(envPathPtr != NULL)
}//PathInfo::PathInfo()
