/*****************************************************************************/
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/




#ifndef CF_INFO_HH
#define CF_INFO_HH

#include "appinfo.hh"
#include <string>
#include <list>

using namespace std;

#ifndef APP_NAME
#define APP_NAME "APP_NEEDS_NAME"
#endif

#ifndef INSTALL_DIR_BASE
#define INSTALL_DIR_BASE "/usr/local"	
#endif

#ifndef PATH_ENV_VAR
#define PATH_ENV_VAR "PATH"	
#endif

/**Deals with file paths.  Use BuildPathName() to search for a file, the returned string
 	will be the filepath + filename.  This class is a singleton so use GetInstance() instead
	of creating it yourself.
*/
class PathInfo
{
public:

	///Gets an instance
	static PathInfo& GetInstance();
	
	/**Prepends given paths to search order
		@param pathStr A string containing paths to add (paths delimited by ":")
	*/
	void AddPaths(const string& pathStr);
	
	/**Searchs for the given file in all paths (checks current working directory first)
		@param filename file to search for
		@return Stringg containing full path + filename, empty string if file not found
	*/
	string BuildPathName(const string& filename);
	
	///Gets the home directory of the current user
	string GetHomeDir() {return homeDir;}
	
	///Gets the app's home directory in the current user's home directory
	string GetAppHomeDir() {return appHomeDir;}
	
	///Gets the directory prefix that the app was installed in
	string GetInstallDir() {return installDir;}
	
	///Gets the path search list
	const list<string>& GetPaths() {return paths;}
	
	//Gets the path search list in string form (paths delimited by ":")
	string GetPathString();

private:	
	PathInfo();
	
	string homeDir;
	string appHomeDir;
	string installDir;
	list<string> paths;
};//class CF_Info


#endif
