/*****************************************************************************/
//  win.cc
//  Copyright (C) 1999 Brett Hall <swizin@rain.org>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or 
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA 
/*****************************************************************************/


//////////////////////////////////////////////////
// win.cc
//	-Main window for controlFreak test-bed
//
//////////////////////////////////////////////////


#include "win.hh"
#include <gtk--/main.h>
#include <unistd.h>


//////////////////////////////////////////////////
//BEGIN MainWin 

MainWin::MainWin(const string& script) : setWindowTitle(*this)
{
	//connect signals
	connect_to_method(delete_event, this, &MainWin::on_delete_event);
	
	//set window props
	set_name("MainWin");
	set_wmclass(script, "ControlFreak");
	set_policy(false, false, true);
	realize();
	
	//add commands to mgr
	cmds.RegisterFunction(&setWindowTitle);
	cmds.RegisterFunction(&printDir);
	cmds.RegisterFunction(&changeDir);
	
	//create widget
	CF_OptionMgr options;
	options.AddOption("TITLE");
	options.AddOption("POSITION");
	cfWidget = new CF_Widget(script, *this, &cmds, &options);	
	if(options.QueryOption("TITLE"))
	{
		set_title(options.GetArg("TITLE"));
	}//if(options.QueryOption("TITLE"))
	else
	{
		set_title("ControlFreak");
	}//else
	if(options.QueryOption("POSITION"))
	{
		string posStr = options.GetArg("POSITION");
		unsigned int pos = posStr.find('+');
		if(pos == string::npos)
			cerr << "Bad position string" << endl;
		else
		{
			string xStr = posStr.substr(0, pos);
			string yStr = posStr.substr(pos + 1, posStr.length() - pos);
			int x = atoi(xStr.c_str());
			int y = atoi(yStr.c_str());
			set_uposition(x, y);
		}//else
	}//if(options.QueryOption("POSITION"))
	
	Gtk_Widget& gtkWidget = cfWidget->GetPackableWidget();
	gtkWidget.show_all();
	add(gtkWidget);
	show();
	hide();
	show();	//have to do this little dance to get window to have right width for some reason
}//MainWin::MainWin()


gint MainWin::on_delete_event(GdkEventAny *ev)
{
	//Gtk_Main::instance()->quit();
	gtk_main_quit();
	return 0;
}//gint MainWin::on_delete_event(GdkEventAny *ev)


MainWin::~MainWin()
{
	if(cfWidget != NULL)
		delete cfWidget;
}//MainWin::~MainWin()


//END MainWin 

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN SetWindowTitle

string SetWindowTitle::Call(vector<string>& args)
{
	win.set_title(args[0]);
	return "";
}//string SetWindowTitle::Call(vector<string>& args)

//END SetWindowTitle


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN PrintDir

string PrintDir::Call(vector<string>& args)
{
	char buffer[256];
	char* ptr = getcwd(buffer, 256);
	if(ptr != NULL)
		return buffer;		
	else
		return "";

}//string PrintDir::Call(vactor<string> args)

//END PrintDir


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//BEGIN ChangeDir

ChangeDir::ChangeDir() : CF_CallBack("ChangeDir", 0), fileSelect("Select Directory")
{
	//hook up OK button signal in fileSelect
	connect_to_method(fileSelect.get_ok_button()->clicked, this, &ChangeDir::OKClicked);
	connect_to_method(fileSelect.get_cancel_button()->clicked, this, &ChangeDir::CancelClicked);
	connect_to_method(fileSelect.delete_event, this, &ChangeDir::DeleteEvent);
}//ChangeDir()

string ChangeDir::Call(vector<string>& args)
{
	if(!args.empty() && !args[0].empty())
	{
		chdir(args[0].c_str());
	}//if(args.size > 0)
	else
	{
		fileSelect.show();
	}//else
	
	return "";
}//string Call(vector<string>& args)

void ChangeDir::OKClicked()
{
	fileSelect.hide();
	string filename = fileSelect.get_filename();
	filename = filename.substr(0, filename.find_last_of("/"));
	cout << "changing to directory " << filename << endl;
	chdir(filename.c_str());
}//void OKClicked()
	
void ChangeDir::CancelClicked()
{
	fileSelect.hide();
}//void ChangeDir::CancelClicked()
	
gint ChangeDir::DeleteEvent(GdkEventAny* ev)
{
	fileSelect.hide();
	return TRUE;
}//gint ChangeDir::DeleteEvent(GdkEventAny* ev)


//END ChangeDir



//BEGIN GLOBALS

//END GLOBALS


